clc

% This is the program for the estimation of rank dependent utility model
% with three model parameters:
%      (loss aversion or loving): -1/3 <= gamma <= 1/3
%      (ambiguity aversion or loving): -1/3 <= beta <= 1/3
%      (risk aversion): 0 <= rho <= 5
% An addition restriction: -1/3 <= gamma + beta <= 1/3

load AllData.mat;

% AllData is (50xN)x8 matrix of the experimental data 
% from Ahn, D., S. Choi, D. Gale, and S. Kariv (2013),
% where N is the number of subjects.
% AllData = [ID Round D1 D2 D3 Intercept1 Intercept2 Intercept3]
%       where Di is a token allocation at ith state.
% state 2 is unambiguous.

dt = AllData;

n = length(dt)/50; 

% nr is the number of replication in the bootstrap method of computing
%    the standard errors of model parameters.

nr = 500;

Estm = []; PreD = []; bootsample = [];

for i = 1:n
        
    m1 = 50*(i-1)+1; m2 = 50*i; 
    id = dt(m1,1)  
    
    % y and ymax are defined as demand and intercept at unambiguous state
    
    x = dt(m1:m2,3); y = dt(m1:m2,4); z = dt(m1:m2,5);
    xmax = dt(m1:m2,6); ymax = dt(m1:m2,7); zmax = dt(m1:m2,8);
    
    % Parameter Restrictions; eps = 0.01
    % -1/3 + eps <= gamma <= 1/3 - eps, -1/3 + eps <= beta <= 1/3 - eps, 0 <= rho <= 5
    % -1/3 + eps <= gamma + beta <= 1/3 - eps
    
    eps = 0.01;
    
%    A = [-1 0 0; 1 0 0; 0 -1 0; 0 1 0; -1 -1 0; 1 1 0; 0 0 -1; 0 0 1];
%    b = [1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 0; 5];

    A = [-1 0 0; 1 0 0; 0 -1 0; 0 1 0; -1 -1 0; 1 1 0; 0 0 -1; 0 0 1];
    b = [1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 1/3 - eps; 0; 5];
        
    LB = [ -1/3 + eps; -1/3 + eps; 0]; UB = [ 1/3 - eps; 1/3 - eps; 5];
       
    % Stage 1: estimating individual model parameters
    
    options = optimset('Algorithm','interior-point','Display','iter','MaxFunEvals',1.0E+5);
    
    pm01 = [0;0;0.05];
    
    [pm1,tssr1] = fmincon(@ssr1,pm01,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
        
    pm02 = [1/6;0;0.05];
    
    [pm2,tssr2] = fmincon(@ssr1,pm02,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm03 = [0;1/6;0.05];
    
    [pm3,tssr3] = fmincon(@ssr1,pm03,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
        
    pm04 = [-1/6; 0; 0.05];
    
    [pm4,tssr4] = fmincon(@ssr1,pm04,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm05 = [0; -1/6; 0.05];
    
    [pm5,tssr5] = fmincon(@ssr1,pm05,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm06 = [0;0; 0.5];
    
    [pm6,tssr6] = fmincon(@ssr1,pm06,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm07 = [0.02; 0.02;0.0001]; % important for ID 314, 413
    
    [pm7,tssr7] = fmincon(@ssr1,pm07,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm08 = [-0.01; -0.15; 0.01]; % important for ID 620
    
    [pm8,tssr8] = fmincon(@ssr1,pm08,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm09 = [-0.1; 0.1; 0.039]; % important for ID
    
    [pm9,tssr9] = fmincon(@ssr1,pm09,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm10 = [0.1; 0.2; 0.01]; 
    
    [pm10,tssr10] = fmincon(@ssr1,pm10,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    pm11 = [0.05; -0.05; 0.05]; 
    
    [pm11,tssr11] = fmincon(@ssr1,pm11,A,b,[],[],LB,UB,[],options,y,x,z,ymax,xmax,zmax);
    
    tssrmin = min([tssr1 tssr2 tssr3 tssr4 tssr5 tssr6 tssr7 tssr8 tssr9 tssr10 tssr11]);
    
    if (tssr1 <= tssrmin)
        
        pm = pm1; tssr = tssrmin;
    
    elseif (tssr2 <= tssrmin)
        
        pm = pm2; tssr = tssrmin;
        
    elseif (tssr3 <= tssrmin)
        
        pm = pm3; tssr = tssrmin;
        
    elseif (tssr4 <= tssrmin)
        
        pm = pm4; tssr = tssrmin;
        
    elseif (tssr5 <= tssrmin)
        
        pm = pm5; tssr = tssrmin;
        
    elseif (tssr6 <= tssrmin) 
        
        pm = pm6; tssr = tssrmin;

    elseif (tssr7 <= tssrmin) 
        
        pm = pm7; tssr = tssrmin;

    elseif (tssr8 <= tssrmin) 
        
        pm = pm8; tssr = tssrmin;        
        
    elseif (tssr9 <= tssrmin) 
        
        pm = pm9; tssr = tssrmin;

    elseif (tssr10 <= tssrmin) 
        
        pm = pm10; tssr = tssrmin;        
    
    else 
        
        pm = pm11; tssr = tssrmin;

    end
    
    beta = pm(1); gamma = pm(2); rho = pm(3);
    
    Estm = [Estm; i,id,beta, gamma,rho, tssr];
    
    % Stage 2: Estimating the covariance matrix using the bootstrap method
    
    btsample = bstrap(pm,y,x,z,ymax,xmax,zmax,nr);
    
    bootsample = [bootsample; id*ones(nr,1),btsample];
    
    %%% To construct the Wald statistic for the null hypothesis that alpha = 0 and gamma = 0
    
    cov_betas = cov(btsample(:,1:2));
    var_rho = cov(btsample(:,3));

    Estm = [Estm;i,id,beta,gamma,rho,cov_betas(1,1),cov_betas(2,2),cov_betas(2,1),var_rho,tssr];
    
    % Stage 3: Computing predicted demands using the estimated model
    %          parameters
        
    for j = 1:50
    
        p1 = 1/xmax(j); p2 = 1/ymax(j); p3 = 1/zmax(j);
        
        pd = demand(p1,p2,p3,gamma,beta,rho);
    
        PreD = [PreD;i,id,j,pd(1),pd(2),pd(3),xmax(j),ymax(j),zmax(j)];
        
    end
    
save ESTM_RDU_Jan312013 Estm
save PreD_RDU_Jan312013 PreD
save Bootsample_RDU_Jan312013 bootsample
        
end    




