clc

% Matlab version R2009a is used.

% This is the program for the estimation of REU model
% There are two model parameters: 
%      ambiguity aversion:  0 <= alpha <= 2
%      (constant absolute) risk aversion:   0 <= rho <= 0.5

load AllData.mat;

% AllData is (50xN)x8 matrix of the experimental data 
% from Ahn, D., S. Choi, D. Gale, and S. Kariv (2010),
% where N is the number of subjects.
% AllData = [ID Round D1 D2 D3 Intercept1 Intercept2 Intercept3]
%       where Di is a token allocation at ith state.
% state 2 is unambiguous.

dt = AllData;

n = length(dt)/50; 

nr = 100; % the number of replications in the bootstrap method of computing standard errors

Estm = []; PreD = [];

for i = 1:n
    
    m1 = 50*(i-1)+1; m2 = 50*i; 
    id = dt(m1,1) 
    i
        
    % y and ymax are defined as demand and intercept at unambiguous state
    
    x = dt(m1:m2,3); y = dt(m1:m2,4); z = dt(m1:m2,5);
    xmax = dt(m1:m2,6); ymax = dt(m1:m2,7); zmax = dt(m1:m2,8);
    
    % Stage 1: estimating individual model parameters
    
    pm01 = [4;3];
    
    [pm1,tssr1] = fminsearch(@ssr,pm01,optimset('Display','iter'),y,x,z,ymax,xmax,zmax);
    
    pm02 = [-3;-2];
    
    [pm2,tssr2] = fminsearch(@ssr,pm02,optimset('Display','iter'),y,x,z,ymax,xmax,zmax);
    
    pm03 = [0;2];
    
    [pm3,tssr3] = fminsearch(@ssr,pm03,optimset('Display','iter'),y,x,z,ymax,xmax,zmax);
    
    pm04 = [2;0];
    
    [pm4,tssr4] = fminsearch(@ssr,pm04,optimset('Display','iter'),y,x,z,ymax,xmax,zmax);

    tssrmin = min([tssr1 tssr2 tssr3 tssr4]);
    
    if (tssr1 == tssrmin)
        
        pm = pm1; tssr = tssr1;
    
    elseif (tssr2 == tssrmin)
        
        pm = pm2; tssr = tssr2;
        
    elseif (tssr3 == tssrmin)
        
        pm = pm3; tssr = tssr3;
    else
        
        pm = pm4; tssr = tssr4;
    end
    
    alpha = 2/(1+exp(pm(1))); rho = 0.5/(1+exp(pm(2)));
    
    % Stage 2: Computing predicted demands using the estimated model
    %          parameters
    
    for j = 1:50
    
        p1 = 1/xmax(j); p2 = 1/ymax(j); p3 = 1/zmax(j);
        
        pd = demand(p1,p2,p3,alpha,rho);
    
        PreD = [PreD;id,pd(1),pd(2),pd(3),xmax(j),ymax(j),zmax(j)];
    end
    
    se = bstrap(y,x,z,ymax,xmax,zmax,nr);
    
    % Stage 3: Computing the standard errors using the bootstrap method
        
    Estm = [Estm;i,id,alpha,se(1),rho,se(2),tssr];
    
    save ESTM_REU Estm
    save PreD_REU PreD
    
end    





