clc

% Matlab version R2009a is used.
% This is the program for the simulation of rank dependent utility model
% with three model parameters:
%      (Probability weighting-pessimim/optimism): -1/3 <= gamma <= 1/3
%      (ambiguity attitude): -1/3 <= beta <= 1/3
%      (risk aversion): 0 <= rho <= 5
% An addition restriction: -1/3 <= gamma + beta <= 1/3
% We sometimes use beta and delta interchangeably

M = 250; zmax = 40;

%M = 300; zmax = 35;

dt = []; n = 50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the budget lines whose log(p_i/p_j) ranges between -2.5 and 2.5
% while keeping the level of wealth constant
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:(n+1)
    
    d = (i-1)*(5/n) - 2.5; % generating logprice b/w -2.5 and 2.5
    
    xmax = ((1+exp(d))*(M/zmax) + sqrt(((M/zmax)*(1+exp(d)))^2 + 4*exp(d)*M))/(2*exp(d));
    ymax = exp(d)*xmax;
    
    % xmax and ymax are the intercepts of assets of interest
    
    dt = [dt;zmax,ymax,xmax]; 
        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

xmax = dt(:,3); ymax = dt(:,2); zmax = dt(:,1);

% Choose the parameter values of gamma and beta

gamma = 0; beta = 0.2;  % -1/3 <= gamma + beta <= 1/3

rhovec = [0.05,0.10,0.25,0.5,1,5]; % the set of rhos used in the diagram
   
pred1 = []; pred2 = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing relative demands between state 1 (ambiguous) and 2 (risky)
%     and between state 1 and 3 (two ambiguous states)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 1:6
    
    rho = rhovec(k);

    for i = 1:(n+1)
            
        pd = demand(1/xmax(i),1/ymax(i),1/zmax(i),gamma,beta,rho); % x1 and x2
        
        pred1 = [pred1;xmax(i),ymax(i),zmax(i),pd(1),pd(2),pd(3)];     
    
        pd2 = demand(1/xmax(i),1/zmax(i),1/ymax(i),gamma,beta,rho); % x1 and x3
    
        pred2 = [pred2;xmax(i),zmax(i),ymax(i),pd2(1),pd2(2),pd2(3)]; 
            
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the simulated relation between log(p1/p2) and x1/(x1+x2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
a = zeros(101,1); a1 = []; a2 = []; e1 = []; e2 = []; t2 = [];

b1 = []; b2 = [];

for i = 1:101
    
    d1 = 0.01*(i-1); d2 = 0.05*(i-1) - 2.5;
    a1 = [a1;d1]; a2 = [a2;d2];
    
end

for k = 1:6
    
    m1 = 51*(k-1)+1; m2 = 51*k;
    
    x = pred1(m1:m2,4); y = pred1(m1:m2,5);
    px1 = x./(x + y); pr1 = log(ymax./xmax);

    if (k == 1)

        plot(pr1,px1,'-b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        
        set(gcf,'Color',[1,1,1])
        box on
        hold on
        
    elseif (k == 2)

        plot(pr1,px1,'-.b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 3)

        plot(pr1,px1,'.b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 4)

        plot(pr1,px1,'-r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on

    elseif (k == 5)

        plot(pr1,px1,'-.r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 6)

        plot(pr1,px1,'.r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
    end
    
end

legend('rho = 0.05', 'rho = 0.10', 'rho = 0.25', 'rho = 0.5', 'rho = 1' , 'rho = 5')

plot(a2,a+0.5,'-.g')
hold on

plot(a,a1,'-.g')
hold off

title('Relation between x_1/(x_1+x_2) and log(p_1/p_2) in the RDU: (gamma,delta)= (0, 0.2)')
xlabel('log(p_1/p_2)')
ylabel('x_1/(x_1+x_2)')   

pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the simulated relation between log(p1/p3) and x1/(x1+x3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 1:6
    
    m1 = 51*(k-1)+1; m2 = 51*k;
    
    x = pred2(m1:m2,4); y = pred2(m1:m2,6);
    px1 = x./(x + y); pr1 = log(ymax./xmax);

    if (k == 1)

        plot(pr1,px1,'-b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        
        set(gcf,'Color',[1,1,1])
        box on
        hold on
        
    elseif (k == 2)

        plot(pr1,px1,'-.b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 3)

        plot(pr1,px1,'.b','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 4)

        plot(pr1,px1,'-r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on

    elseif (k == 5)

        plot(pr1,px1,'-.r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
        
    elseif (k == 6)

        plot(pr1,px1,'.r','LineWidth',1.5);
        axis([-2.5 2.5 0 1]);
        hold on
    end
    
end

legend('rho = 0.05', 'rho = 0.10', 'rho = 0.25', 'rho = 0.5', 'rho = 1' , 'rho = 5')

plot(a2,a+0.5,'-.g')
hold on

plot(a,a1,'-.g')
hold off

title('Relation between x_1/(x_1+x_3) and log(p_1/p_3) in the RDU: (gamma,delta)= (0, 0.2)')
xlabel('log(p_1/p_3)')
ylabel('x_1/(x_1+x_3)')   


    



