function output = demand(p1,p2,p3,gamma,beta,rho)

% Characterizing the demand function
% rho: risk aversion, gamma : loss aversion, beta: ambiguity aversion
% -1/3 <= gamma <= 1/3; -1/3 <= beta <= 1/3; -1/3 <= gamma + beta <= 1/3

pmin = min(p1,p3); pmax = max(p1,p3);

eps = 1.0E-5;

if (rho > eps)
    
    if (gamma <= 1/3 - eps)&&(beta <= 1/3 - eps)
        
        if (gamma + beta <= 1/3 - eps)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if (((p2/pmin) <= ((1/3+gamma)/(1/3-gamma-beta)))&&((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)))&&(((p2+pmax)/pmin) <= ((2/3+gamma+beta))/(1/3-gamma-beta)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            elseif (((p2/(p1+p3)) > ((1/3+gamma)/(2/3-gamma)))&&((pmax/pmin) <= (1/3+beta)/(1/3-gamma-beta))) % x1 = x3 > x2
                s1 = 1/(p1+p2+p3) + (p2/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                s3 = s1;
                s2 = 1/(p1+p2+p3) - ((p1+p3)/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                if (s2 >= 0)
                    x11 = s1; x21 = s2; x31 = s3;
                else
                    x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
                end
            elseif ((((p2+pmax)/pmin) > ((2/3+gamma+beta)/(1/3-gamma-beta)))&&((p2/pmax) <= ((1/3+gamma)/(1/3+beta)))) % x2 = xmin < xmax
                s2 = 1/(p1+p2+p3) - (pmin/(rho*(p1+p2+p3)))*(log((p2+pmax)/pmin) - log((2/3+gamma+beta)/(1/3-gamma-beta)));
                smin = s2;
                smax = 1/(p1+p2+p3) + ((p2+pmax)/(rho*(p1+p2+p3)))*(log((p2+pmax)/pmin) - log((2/3+gamma+beta)/(1/3-gamma-beta)));
                if (pmin == p1)              
                    if (s2 >= 0)
                        x21 = s2; x31 = smin; x11 = smax;
                    else                
                        x21 = 0; x31 = 0; x11 = 1/p1; 
                    end          
                else
                    if (s2 >= 0)
                        x21 = s2; x11 = smin; x31 = smax;
                    else
                        x21 = 0; x11 = 0; x31 = 1/p3;
                    end                          
                end
            else % xmax > xmin > x2
                s2 = 1/(p1+p2+p3) - (pmax/(rho*(p1+p2+p3)))*(log(p2/pmax) - log((1/3+gamma)/(1/3+beta))) - (pmin/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3+gamma)/(1/3-gamma-beta)));
                smin = 1/(p1+p2+p3) + ((p2+pmin)/(rho*(p1+p2+p3)))*(log(p2/pmax) - log((1/3+gamma)/(1/3+beta))) - (pmin/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3+gamma)/(1/3-gamma-beta)));
                smax = 1/(p1+p2+p3) - (pmax/(rho*(p1+p2+p3)))*(log(p2/pmax) - log((1/3+gamma)/(1/3+beta))) + ((pmax+p2)/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3+gamma)/(1/3-gamma-beta)));
                if (pmin == p1)
                    if (smin <= 0)
                        x11 = 1/p1; x21 = 0; x31 = 0;
                    elseif ((smin > 0) && (s2 <= 0))
                        rmin = 1/(p1+p3) - (pmin/(rho*(p1+p3)))*(log(pmax/pmin) - log((1/3+beta)/(1/3-gamma-beta)));
                        rmax = 1/(p1+p3) + (pmax/(rho*(p1+p3)))*(log(pmax/pmin) - log((1/3+beta)/(1/3-gamma-beta)));
                        if (rmin >= 0)
                            x11 = rmax; x21 = 0; x31 = rmin;
                        else
                            x11 = 1/p1; x21 = 0; x31 = 0;
                        end
                    else
                        x11 = smax; x21 = s2; x31 = smin;
                    end
                else
                    if (smin <= 0)
                        x11 = 0; x21 = 0; x31 = 1/p3;
                    elseif ((smin > 0) && (s2 <= 0))
                        rmin = 1/(p1+p3) - (pmin/(rho*(p1+p3)))*(log(pmax/pmin) - log((1/3+beta)/(1/3-gamma-beta)));
                        rmax = 1/(p1+p3) + (pmax/(rho*(p1+p3)))*(log(pmax/pmin) - log((1/3+beta)/(1/3-gamma-beta)));
                        if (rmin >= 0)
                            x11 = rmin; x21 = 0; x31 = rmax;
                        else
                            x11 = 0; x21 = 0; x31 = 1/p3;
                        end
                    else
                        x11 = smin; x21 = s2; x31 = smax;
                    end
                end          
            end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
            if (((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-gamma-beta)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&(((p2+pmax)/pmin) <= ((2/3+gamma+beta)/(1/3-gamma-beta)))) % x1=x2=x3
                x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
            elseif ((((p2+pmax)/pmin) > ((2/3+gamma+beta)/(1/3-gamma-beta)))&&((pmax/p2) <= ((1/3+gamma+beta)/(1/3)))) % x2 = xmin < xmax
                s2 = 1/(p1+p2+p3) - (pmin/(rho*(p1+p2+p3)))*(log((pmax+p2)/pmin) - log((2/3+gamma+beta)/(1/3-gamma-beta)));
                smin = s2;
                smax = 1/(p1+p2+p3) + ((p2+pmax)/(rho*(p1+p2+p3)))*(log((pmax+p2)/pmin) - log((2/3+gamma+beta)/(1/3-gamma-beta)));
                if (pmin == p1)                
                    if (s2 >= 0)
                        x12 = smax; x22 = s2; x32 = smin;
                    else
                        x12 = 1/p1; x22 = 0; x32 = 0;
                    end
                else
                    if (s2 >= 0)
                        x12 = smin; x22 = s2; x32 = smax;
                    else
                        x12 = 0; x22 = 0; x32 = 1/p3;
                    end
                end
            elseif (((pmax / (p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((p2/pmin) <= ((1/3)/(1/3-gamma-beta)))) % xmax = x2 > xmin
                s2 = 1/(p1+p2+p3) + (pmax/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                smax = s2;
                smin = 1/(p1+p2+p3) - ((p2 + pmin)/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                if (pmin == p1)
                    if (smin >= 0)
                        x12 = smax; x22 = s2; x32 = smin;
                    else
                        x12 = 1/(p1+p2); x22 = 1/(p1+p2); x32 = 0;
                    end
                else
                    if (smin >= 0)
                        x12 = smin; x22 = s2; x32 = smax;
                    else
                        x12 = 0; x22 = 1/(p2+p3); x32 = 1/(p2+p3);
                    end
                end
            else % xmax > x2 > xmin
                smin = 1/(p1+p2+p3) - ((p2+pmin)/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3))) - (pmin/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3)/(1/3-gamma-beta)));
                s2 = 1/(p1+p2+p3) + ((pmax)/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3))) - (pmin/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3)/(1/3-gamma-beta)));
                smax = 1/(p1+p2+p3) + ((pmax)/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3))) + ((pmax+p2)/(rho*(p1+p2+p3)))*(log(p2/pmin) - log((1/3)/(1/3-gamma-beta)));
                if (pmin == p1)
                    if (s2 <= 0)
                        x12 = 1/p1; x22 = 0; x32 = 0;                    
                    elseif (s2 > 0) && (smin <= 0)
                        r2 = 1/(p2+p1) - (p1/(rho*(p2+p1)))*(log(p2/p1)-log((1/3)/(1/3-gamma-beta)));
                        rmax = 1/(p2+p1) + (p2/(rho*(p2+p1)))*(log(p2/p1)-log((1/3)/(1/3-gamma-beta)));
                        if (r2 >= 0)
                            x12 = rmax; x22 = r2; x32 = 0;
                        else
                            x12 = 1/p1; x22 = 0; x32 = 0;
                        end
                    else
                        x12 = smax; x22 = s2; x32 = smin;
                    end
                else
                    if (s2 <= 0)
                        x12 = 0; x22 = 0; x32 = 1/p3;                    
                    elseif (s2 > 0) && (smin <= 0)
                        r2 = 1/(p2+p3) - (p3/(rho*(p2+p3)))*(log(p2/p3)-log((1/3)/(1/3-gamma-beta)));
                        rmax = 1/(p2+p3) + (p2/(rho*(p2+p3)))*(log(p2/p3)-log((1/3)/(1/3-gamma-beta)));
                        if (r2 >= 0)
                            x12 = 0; x22 = r2; x32 = rmax;
                        else
                            x12 = 0; x22 = 0; x32 = 1/p3;
                        end
                    else
                        x12 = smin; x22 = s2; x32 = smax;
                    end
                end
            end
        
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if (((pmax/p2) <= ((1/3+gamma+beta)/(1/3-gamma)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma))))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            elseif ((((p1+p3)/p2) > ((2/3+gamma)/(1/3-gamma)))&&((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-beta)))) % x1 = x3 < x2
                s1 = 1/(p1+p2+p3) - (p2/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma))); 
                s3 = s1;
                s2 = 1/(p1+p2+p3) + ((p1+p3)/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma)));
                if (s1 >= 0)
                    x13 = s1; x23 = s2; x33 = s3;
                else
                    x13 = 0; x23 = 1/p2; x33 = 0;
                end
            elseif (((pmax/(p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((pmin/p2) <= ((1/3-beta)/(1/3-gamma)))) % x2 = xmax > xmin
                smin = 1/(p1+p2+p3) - ((p2+pmin)/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                smax = 1/(p1+p2+p3) + ((pmax)/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                s2 = smax;
                if (pmin == p1)
                    if (smin >= 0)
                        x13 = smax; x23 = s2; x33 = smin;
                    else
                        x13 = 1/(p1+p2); x23 = 1/(p1+p2); x33 = 0;
                    end
                else
                    if (smin >= 0)
                        x13 = smin; x23 = s2; x33 = smax;
                    else
                        x13 = 0; x23 = 1/(p2+p3); x33 = 1/(p2+p3);
                    end
                end
            else % x2 > xmax > xmin
                s2 = 1/(p1+p2+p3) + (pmax/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) + (pmin/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                smax = 1/(p1+p2+p3) + (pmax/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) - ((p2+pmax)/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                smin = 1/(p1+p2+p3) - ((p2+pmin)/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) + (pmin/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                if (pmin == p1)
                    if (smax < 0)
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                    elseif (smax >= 0) && (smin < 0)
                        r2 = 1/(p2+pmin) + (pmin/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        rmax = 1/(p2+pmin) - (p2/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        if (rmax >= 0)
                            x13 = rmax; x23 = r2; x33 = 0;
                        else
                            x13 = 0; x23 = 1/p2; x33 = 0;
                        end                   
                    else %(smin >= 0)
                        x13 = smax; x23 = s2; x33 = smin;
                    end
                else
                    if (smax < 0)
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                    elseif (smax >= 0) && (smin < 0)
                        r2 = 1/(p2+pmin) + (pmin/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        rmax = 1/(p2+pmin) - (p2/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        if (rmax >= 0)
                            x13 = 0; x23 = r2; x33 = rmax;
                        else
                            x13 = 0; x23 = 1/p2; x33 = 0;
                        end                   
                    else %(smin >= 0)
                        x13 = smin; x23 = s2; x33 = smax;
                    end
                end
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
            
        else
%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if ((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            elseif ((p2/(p1+p3)) > ((1/3+gamma)/(2/3-gamma))) % x1 = x3 > x2
                s1 = 1/(p1+p2+p3) + (p2/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                s3 = s1;
                s2 = 1/(p1+p2+p3) - ((p1+p3)/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                if (s2 >= 0)
                    x11 = s1; x21 = s2; x31 = s3;
                else
                    x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
                end
            end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
                x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if (((pmax/p2) <= ((1/3+gamma+beta)/(1/3-gamma)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma))))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            elseif ((((p1+p3)/p2) > ((2/3+gamma)/(1/3-gamma)))&&((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-beta)))) % x1 = x3 < x2
                s1 = 1/(p1+p2+p3) - (p2/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma))); 
                s3 = s1;
                s2 = 1/(p1+p2+p3) + ((p1+p3)/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma)));
                if (s1 >= 0)
                    x13 = s1; x23 = s2; x33 = s3;
                else
                    x13 = 0; x23 = 1/p2; x33 = 0;
                end
            elseif (((pmax/(p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((pmin/p2) <= ((1/3-beta)/(1/3-gamma)))) % x2 = xmax > xmin
                smin = 1/(p1+p2+p3) - ((p2+pmin)/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                smax = 1/(p1+p2+p3) + ((pmax)/(rho*(p1+p2+p3)))*(log(pmax/(p2+pmin)) - log((1/3+gamma+beta)/(2/3-gamma-beta)));
                s2 = smax;
                if (pmin == p1)
                    if (smin >= 0)
                        x13 = smax; x23 = s2; x33 = smin;
                    else
                        x13 = 1/(p1+p2); x23 = 1/(p1+p2); x33 = 0;
                    end
                else
                    if (smin >= 0)
                        x13 = smin; x23 = s2; x33 = smax;
                    else
                        x13 = 0; x23 = 1/(p2+p3); x33 = 1/(p2+p3);
                    end
                end
            else % x2 > xmax > xmin
                s2 = 1/(p1+p2+p3) + (pmax/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) + (pmin/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                smax = 1/(p1+p2+p3) + (pmax/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) - ((p2+pmax)/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                smin = 1/(p1+p2+p3) - ((p2+pmin)/(rho*(p1+p2+p3)))*(log(pmax/p2) - log((1/3+gamma+beta)/(1/3-gamma))) + (pmin/(rho*(p1+p2+p3)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                if (pmin == p1)
                    if (smax < 0)
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                    elseif (smax >= 0) && (smin < 0)
                        r2 = 1/(p2+pmin) + (pmin/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        rmax = 1/(p2+pmin) - (p2/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        if (rmax >= 0)
                            x13 = rmax; x23 = r2; x33 = 0;
                        else
                            x13 = 0; x23 = 1/p2; x33 = 0;
                        end                   
                    else %(smin >= 0)
                        x13 = smax; x23 = s2; x33 = smin;
                    end
                else
                    if (smax < 0)
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                    elseif (smax >= 0) && (smin < 0)
                        r2 = 1/(p2+pmin) + (pmin/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        rmax = 1/(p2+pmin) - (p2/(rho*(p2+pmin)))*(log(pmin/p2) - log((1/3-beta)/(1/3-gamma)));
                        if (rmax >= 0)
                            x13 = 0; x23 = r2; x33 = rmax;
                        else
                            x13 = 0; x23 = 1/p2; x33 = 0;
                        end                   
                    else %(smin >= 0)
                        x13 = smin; x23 = s2; x33 = smax;
                    end
                end
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
                      
        end
        
    elseif (gamma > 1/3 - eps)&&(beta <= 1/3 - eps)
        
        x1 = 1/(p1+p2+p3); x2 = 1/(p1+p2+p3); x3 = 1/(p1+p2+p3);
            
        output = [x1,x2,x3];
        
    elseif (gamma <= 1/3 - eps)&&(beta > 1/3 - eps)

%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if ((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            else % x1 = x3 > x2
                s1 = 1/(p1+p2+p3) + (p2/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                s3 = s1;
                s2 = 1/(p1+p2+p3) - ((p1+p3)/(rho*(p1+p2+p3)))*(log(p2/(p1+p3)) - log((1/3+gamma)/(2/3-gamma)));
                if (s2 >= 0)
                    x11 = s1; x21 = s2; x31 = s3;
                else
                    x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
                end
            end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
                x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if ((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma)))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            else % x1 = x3 < x2
                s1 = 1/(p1+p2+p3) - (p2/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma))); 
                s3 = s1;
                s2 = 1/(p1+p2+p3) + ((p1+p3)/(rho*(p1+p2+p3)))*(log((p1+p3)/p2) - log((2/3+gamma)/(1/3-gamma)));
                if (s1 >= 0)
                    x13 = s1; x23 = s2; x33 = s3;
                else
                    x13 = 0; x23 = 1/p2; x33 = 0;
                end
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
        
    end
    
else 
    
    % rho <= eps
    
    if (gamma <= 1/3 - eps)&&(beta <= 1/3 - eps)
        
        if (gamma + beta <= 1/3 - eps)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if (((p2/pmin) <= ((1/3+gamma)/(1/3-gamma-beta)))&&((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)))&&(((p2+pmax)/pmin) <= ((2/3+gamma+beta))/(1/3-gamma-beta)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            elseif (((p2/(p1+p3)) > ((1/3+gamma)/(2/3-gamma)))&&((pmax/pmin) <= (1/3+beta)/(1/3-gamma-beta))) % x1 = x3 > x2
                    x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
            elseif ((((p2+pmax)/pmin) > ((2/3+gamma+beta)/(1/3-gamma-beta)))&&((p2/pmax) <= ((1/3+gamma)/(1/3+beta)))) % x2 = xmin < xmax
                if (pmin == p1)              
                        x21 = 0; x31 = 0; x11 = 1/p1; 
                else
                        x21 = 0; x11 = 0; x31 = 1/p3;
                end
            else % xmax > xmin > x2
                if (pmin == p1)
                        x11 = 1/p1; x21 = 0; x31 = 0;
                else
                        x11 = 0; x21 = 0; x31 = 1/p3;
                end          
            end
                
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
            if (((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-gamma-beta)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&(((p2+pmax)/pmin) <= ((2/3+gamma+beta)/(1/3-gamma-beta)))) % x1=x2=x3
                x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
            elseif ((((p2+pmax)/pmin) > ((2/3+gamma+beta)/(1/3-gamma-beta)))&&((pmax/p2) <= ((1/3+gamma+beta)/(1/3)))) % x2 = xmin < xmax
                if (pmin == p1)                
                        x12 = 1/p1; x22 = 0; x32 = 0;
                else
                        x12 = 0; x22 = 0; x32 = 1/p3;
                end
            elseif (((pmax / (p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((p2/pmin) <= ((1/3)/(1/3-gamma-beta)))) % xmax = x2 > xmin
                if (pmin == p1)
                        x12 = 1/(p1+p2); x22 = 1/(p1+p2); x32 = 0;
                else
                        x12 = 0; x22 = 1/(p2+p3); x32 = 1/(p2+p3);
                end
            else % xmax > x2 > xmin
                if (pmin == p1)
                        x12 = 1/p1; x22 = 0; x32 = 0;                    
                else
                        x12 = 0; x22 = 0; x32 = 1/p3;                    
                end
            end
        
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if (((pmax/p2) <= ((1/3+gamma+beta)/(1/3-gamma)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma))))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            elseif ((((p1+p3)/p2) > ((2/3+gamma)/(1/3-gamma)))&&((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-beta)))) % x1 = x3 < x2
                    x13 = 0; x23 = 1/p2; x33 = 0;
            elseif (((pmax/(p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((pmin/p2) <= ((1/3-beta)/(1/3-gamma)))) % x2 = xmax > xmin
                if (pmin == p1)
                        x13 = 1/(p1+p2); x23 = 1/(p1+p2); x33 = 0;
                else
                        x13 = 0; x23 = 1/(p2+p3); x33 = 1/(p2+p3);
                end
            else % x2 > xmax > xmin
                if (pmin == p1)
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                else
                        x13 = 0; x23 = 1/p2; x33 = 0;                    
                end
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
            
        else
%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if ((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            else % x1 = x3 > x2
                    x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
            end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
                x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if (((pmax/p2) <= ((1/3+gamma+beta)/(1/3-gamma)))&&((pmax/(p2+pmin)) <= ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma))))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            elseif ((((p1+p3)/p2) > ((2/3+gamma)/(1/3-gamma)))&&((pmax/pmin) <= ((1/3+gamma+beta)/(1/3-beta)))) % x1 = x3 < x2
                    x13 = 0; x23 = 1/p2; x33 = 0;
            elseif (((pmax/(p2+pmin)) > ((1/3+gamma+beta)/(2/3-gamma-beta)))&&((pmin/p2) <= ((1/3-beta)/(1/3-gamma)))) % x2 = xmax > xmin
                if (pmin == p1)
                        x13 = 1/(p1+p2); x23 = 1/(p1+p2); x33 = 0;
                else
                        x13 = 0; x23 = 1/(p2+p3); x33 = 1/(p2+p3);
                end
            else % x2 > xmax > xmin
                x13 = 0; x23 = 1/p2; x33 = 0;                    
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
                      
        end
        
    elseif (gamma > 1/3 - eps)&&(beta <= 1/3 - eps)
        
        x1 = 1/(p1+p2+p3); x2 = 1/(p1+p2+p3); x3 = 1/(p1+p2+p3);
            
        output = [x1,x2,x3];
        
    elseif (gamma <= 1/3 - eps)&&(beta > 1/3 - eps)

%%%%%%%%%%% SP1 where x2 <= min{x1,x3}

            if ((p2/(p1+p3))<= ((1/3+gamma)/(2/3-gamma)));            
                x11 = 1/(p1+p2+p3); x21 = 1/(p1+p2+p3); x31 = 1/(p1+p2+p3);
            elseif ((p2/(p1+p3)) > ((1/3+gamma)/(2/3-gamma))) % x1 = x3 > x2
                x21 = 0; x11 = 1/(p1 + p3); x31 = x11;
            end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%% SP2 where min{x1,x3} <= x2 <= max{x1,x3}         
        
            x12 = 1/(p1+p2+p3); x22 = x12; x32 = x12;
        
%%%%%%%%%%%%%%%%%%% SP3 where max{x1,x3} <= x2 (pmin > p2)      

            if ((((p1+p3)/p2) <= ((2/3+gamma)/(1/3-gamma)))) % x1=x2=x3
                x13 = 1/(p1+p2+p3); x23 = x13; x33 = x13;
            elseif (((p1+p3)/p2) > ((2/3+gamma)/(1/3-gamma))) % x1 = x3 < x2
                x13 = 0; x23 = 1/p2; x33 = 0;
            end
        
            co11 = 1/3+gamma; co12 = 1/3+beta; co13 = 1/3-gamma-beta;
            co21 = 1/3+gamma+beta; co22 = 1/3; co23 = 1/3-gamma-beta;
            co31 = 1/3+gamma+beta; co32 = 1/3-beta; co33 = 1/3-gamma;
        
            U1 = co11*CARA(x21,rho) + co12*CARA(min(x11,x31),rho) + co13*CARA(max(x11,x31),rho);
            U2 = co21*CARA(min(x12,x32),rho) + co22*CARA(x22,rho) + co23*CARA(max(x12,x32),rho);
            U3 = co31*CARA(min(x13,x33),rho) + co32*CARA(max(x13,x33),rho) + co33*CARA(x23,rho);
        
            if (U1 >= U2) && (U1 >= U3)
            
                output = [x11,x21,x31];
            
            elseif (U3 > U1) && (U3 >= U2)
            
                output = [x13,x23,x33];
            
            else
            
                output = [x12,x22,x32];
            
            end
        
    end
        
end
        
        
            
end