function output = demand(p1,p2,p3,alpha,rho)

% Characterize the optimal demand in the REU model
% We use the numerical method of computing the optimal demand

if ((alpha < 1e-05)&&(rho < 1e-05))

    if ((p1 <= p2)&&(p1 <= p3))
        
        x1 = 1/p1; x2 = 0; x3 = 0;
        
    elseif ((p2 < p1)&&(p2 <= p3))
        
        x1 = 0; x2 = 1/p2; x3 = 0;
        
    else
        
        x1 = 0; x2 = 0; x3 = 1/p3;
        
    end

else 
    
        
    alpha0 = alpha;
          
    options = optimset('Display','off','MaxFunEvals',1e+5,'TolFun',1e-20);
        
    s01 = [log(p2/p1) + 0.001; log(p3/p1) - 0.001];
        
    [s1,fval1] = fminsearch(@negU,s01,options,alpha0,rho,p1,p2,p3);

    s02 = [2.7; -2.7];
        
    [s2,fval2] = fminsearch(@negU,s02,options,alpha0,rho,p1,p2,p3);
    
    s03 = [-2.7; 2.7];
    
    [s3,fval3] = fminsearch(@negU,s03,options,alpha0,rho,p1,p2,p3);
           
    fvalmin = min([fval1 fval2 fval3]);
    
        
    if fval3 == fvalmin
            
        s = s3;       
        
    elseif fval1 == fvalmin
            
        s = s1;
            
    else
            
        s = s2;
                                    
    end
        
    r1 = 1/(1 + exp(s(1)) + exp(s(2))); r2 = 1/(1 + exp(-s(1)) + exp(s(2)-s(1)));
    r3 = max([1-r1-r2, 0]);

    x1 = (1/p1)*r1; x2 = (1/p2)*r2; x3 = (1/p3)*r3;
    
end


output = [x1,x2,x3];

